@@ -92,6 +92,10 @@
 
 #define POPUP_PATH_LAY_OUT			"/popups/background/Before Zoom Items/View Items/Arrange Items"
 
+#define POPUP_PATH_ICON_APPEARANCE		"/popups/selection/Icon Appearance Items"
+#define POPUP_PATH_STRETCH_ICON		        "/popups/selection/Icon Appearance Items/Stretch"
+#define POPUP_PATH_UNSTRETCH_ICON	        "/popups/selection/Icon Appearance Items/Unstretch"
+
 #define COMMAND_PREFIX                          "/commands/"
 #define COMMAND_STRETCH_ICON 			"/commands/Stretch"
 #define COMMAND_UNSTRETCH_ICONS 		"/commands/Unstretch"
@@ -590,6 +594,16 @@ fm_icon_view_supports_keep_aligned (FMIconView *view)
 		 supports_keep_aligned, (view));
 }
 
+static gboolean
+fm_icon_view_supports_labels_beside_icons (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return EEL_CALL_METHOD_WITH_RETURN_VALUE
+		(FM_ICON_VIEW_CLASS, view,
+		 supports_labels_beside_icons, (view));
+}
+
 static void
 update_layout_menus (FMIconView *view)
 {
@@ -946,6 +960,14 @@ real_supports_keep_aligned (FMIconView *view)
 	return FALSE;
 }
 
+static gboolean
+real_supports_labels_beside_icons (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), TRUE);
+
+	return TRUE;
+}
+
 static gboolean
 set_sort_reversed (FMIconView *icon_view, gboolean new_value)
 {
@@ -1022,6 +1044,26 @@ get_default_zoom_level (void)
 	return CLAMP (default_zoom_level, NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
 }
 
+static void
+set_labels_beside_icons (FMIconView *icon_view)
+{
+	gboolean labels_beside;
+
+	if (fm_icon_view_supports_labels_beside_icons (icon_view)) {
+		labels_beside = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
+		
+		if (labels_beside) {
+			nautilus_icon_container_set_label_position
+				(get_icon_container (icon_view), 
+				 NAUTILUS_ICON_LABEL_POSITION_BESIDE);
+		} else {
+			nautilus_icon_container_set_label_position
+				(get_icon_container (icon_view), 
+				 NAUTILUS_ICON_LABEL_POSITION_UNDER);
+		}
+	}
+}
+
 static void
 fm_icon_view_begin_loading (FMDirectoryView *view)
 {
@@ -1079,6 +1121,8 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 		(get_icon_container (icon_view), 
 		 fm_icon_view_get_directory_tighter_layout (icon_view, file));
 
+	set_labels_beside_icons (icon_view);
+
 	/* We must set auto-layout last, because it invokes the layout_changed 
 	 * callback, which works incorrectly if the other layout criteria are
 	 * not already set up properly (see bug 6500, e.g.)
@@ -1313,6 +1357,33 @@ fm_icon_view_get_selection (FMDirectoryView *view)
 	return list;
 }
 
+static void
+count_item (NautilusIconData *icon_data,
+	    gpointer callback_data)
+{
+	guint *count;
+
+	count = callback_data;
+	(*count)++;
+}
+
+static guint
+fm_icon_view_get_item_count (FMDirectoryView *view)
+{
+	guint count;
+
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), 0);
+
+	count = 0;
+	
+	nautilus_icon_container_for_each
+		(get_icon_container (FM_ICON_VIEW (view)),
+		 count_item, &count);
+
+	return count;
+}
+
+
 static void
 set_sort_criterion_by_id (FMIconView *icon_view, const char *id)
 {
@@ -1507,6 +1578,13 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 			(icon_view->details->ui, POPUP_PATH_LAY_OUT, TRUE);
 	}
 
+	nautilus_bonobo_set_hidden
+		(icon_view->details->ui, POPUP_PATH_ICON_APPEARANCE, TRUE);
+
+	nautilus_bonobo_set_hidden
+		(icon_view->details->ui, POPUP_PATH_ICON_APPEARANCE, 
+		 !FM_IS_DESKTOP_ICON_VIEW (view));
+
 	update_layout_menus (icon_view);
 
 	bonobo_ui_component_thaw (icon_view->details->ui, NULL);
@@ -1651,7 +1729,23 @@ icon_container_activate_callback (NautilusIconContainer *container,
 	g_assert (FM_IS_ICON_VIEW (icon_view));
 	g_assert (container == get_icon_container (icon_view));
 
-	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view), file_list);
+	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view),
+					  file_list, 
+					  Nautilus_ViewFrame_OPEN_ACCORDING_TO_MODE, 0);
+}
+
+static void
+icon_container_activate_alternate_callback (NautilusIconContainer *container,
+					    GList *file_list,
+					    FMIconView *icon_view)
+{
+	g_assert (FM_IS_ICON_VIEW (icon_view));
+	g_assert (container == get_icon_container (icon_view));
+
+	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (icon_view), 
+					  file_list, 
+					  Nautilus_ViewFrame_OPEN_ACCORDING_TO_MODE,
+					  Nautilus_ViewFrame_OPEN_FLAG_CLOSE_BEHIND);
 }
 
 static void
@@ -1704,8 +1798,9 @@ play_file (gpointer callback_data)
 	file_uri = nautilus_file_get_uri (file);
 	mime_type = nautilus_file_get_mime_type (file);
 	is_mp3 = eel_strcasecmp (mime_type, "audio/mpeg") == 0;
-	is_ogg = eel_strcasecmp (mime_type, "application/x-ogg") == 0;
-
+	is_ogg = eel_strcasecmp (mime_type, "application/x-ogg") == 0 ||
+                eel_strcasecmp (mime_type, "application/ogg") == 0;
+	
 	mp3_pid = fork ();
 	if (mp3_pid == (pid_t) 0) {
 		/* Set the group (session) id to this process for future killing. */
@@ -1723,22 +1818,24 @@ play_file (gpointer callback_data)
 			}
 			command_str = g_strdup_printf("play -t %s -", suffix);
 		}
+
+		/* read the file with gnome-vfs, feeding it to the sound player's standard input */
+		/* First, open the file. */
+		result = gnome_vfs_open (&handle, file_uri, GNOME_VFS_OPEN_READ);
+		if (result != GNOME_VFS_OK) {
+			_exit (0);
+		}
 			
 		/* since the uri could be local or remote, we launch the sound player with popen and feed it
 		 * the data by fetching it with gnome_vfs
 		 */
 		sound_process = popen(command_str, "w");
 		if (sound_process == 0) {
-			return FALSE;
+			/* Close the file. */
+			result = gnome_vfs_close (handle);			
+			_exit (0);
 		}
 			
-		/* read the file with gnome-vfs, feeding it to the sound player's standard input */
-		/* First, open the file. */
-		result = gnome_vfs_open (&handle, file_uri, GNOME_VFS_OPEN_READ);
-		if (result != GNOME_VFS_OK) {
-			return FALSE;
-		}
-
 		/* allocate a buffer. */
 		buffer = g_malloc(READ_CHUNK_SIZE);
 			
@@ -1764,7 +1861,7 @@ play_file (gpointer callback_data)
 		g_free(buffer);
 		pclose(sound_process);
 		_exit (0);
-	} else {
+	} else if (mp3_pid > (pid_t) 0) {
 		nautilus_sound_register_sound (mp3_pid);
 	}
 		
@@ -2309,6 +2406,18 @@ default_zoom_level_changed_callback (gpointer callback_data)
 	}
 }
 
+static void
+labels_beside_icons_changed_callback (gpointer callback_data)
+{
+	FMIconView *icon_view;
+
+	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
+
+	icon_view = FM_ICON_VIEW (callback_data);
+
+	set_labels_beside_icons (icon_view);
+}
+
 static void
 fm_icon_view_sort_directories_first_changed (FMDirectoryView *directory_view)
 {
@@ -2379,6 +2488,8 @@ create_icon_container (FMIconView *icon_view)
 	
 	g_signal_connect_object (icon_container, "activate",	
 			 G_CALLBACK (icon_container_activate_callback), icon_view, 0);
+	g_signal_connect_object (icon_container, "activate_alternate",	
+			 G_CALLBACK (icon_container_activate_alternate_callback), icon_view, 0);
 	g_signal_connect_object (icon_container, "band_select_started",
 				 G_CALLBACK (band_select_started_callback), icon_view, 0);
 	g_signal_connect_object (icon_container, "band_select_ended",
@@ -2443,9 +2554,11 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 	char *container_uri;
 	char *mime_type;
 	const char *last_slash, *link_name;
-	int n_uris;
+	int n_uris, n_links;
 	gboolean all_local;
 	GArray *points;
+	GdkScreen *screen;
+	int screen_num;
 
 	if (item_uris == NULL) {
 		return;
@@ -2456,8 +2569,9 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 
 	if (eel_vfs_has_capability (container_uri,
 				    EEL_VFS_CAPABILITY_IS_REMOTE_AND_SLOW)) {
-		eel_show_warning_dialog (_("Drag and drop is only supported to local file systems."),
-					 _("Drag and Drop error"),
+		eel_show_warning_dialog (_("Drag and drop is not supported."),
+					 _("Drag and drop is only supported on local file systems."),
+					 _("Drag and Drop Error"),
 					 fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)));
 		g_free (container_uri);
 		return;
@@ -2467,6 +2581,10 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 		action = nautilus_drag_drop_action_ask 
 			(GTK_WIDGET (container),
 			 GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+		if (action == 0) {
+			g_free (container_uri);
+			return;
+		}
 	}
 	
 	/* We don't support GDK_ACTION_ASK or GDK_ACTION_PRIVATE
@@ -2475,8 +2593,9 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 	    (action != GDK_ACTION_COPY) &&
 	    (action != GDK_ACTION_MOVE) &&
 	    (action != GDK_ACTION_LINK)) {
-		eel_show_warning_dialog (_("An invalid drag type was used."),
-					 _("Drag and Drop error"),
+		eel_show_warning_dialog (_("Drag and drop is not supported."),
+					 _("An invalid drag type was used."),
+					 _("Drag and Drop Error"),
 					 fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)));
 		g_free (container_uri);
 		return;
@@ -2484,6 +2603,9 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 
 	point.x = x;
 	point.y = y;
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (view));
+	screen_num = gdk_screen_get_number (screen);
 		
 	/* Most of what comes in here is not really URIs, but rather paths that
 	 * have a file: prefix in them.  We try to sanitize the uri list as a
@@ -2528,6 +2650,7 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 		if (points)
 			g_array_free (points, TRUE);
 	} else {
+		n_links = 0;
 		for (node = real_uri_list; node != NULL; node = node->next) {
 			/* Make a link using the desktop file contents? */
 			uri = node->data;
@@ -2537,7 +2660,8 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 				mime_type = gnome_vfs_get_mime_type (uri);
 
 				if (mime_type != NULL &&
-				    strcmp (mime_type, "application/x-gnome-app-info") == 0) {
+				    (strcmp (mime_type, "application/x-gnome-app-info") == 0 ||
+				     strcmp (mime_type, "application/x-desktop") == 0)) {
 					entry = gnome_desktop_item_new_from_file (path,
 							GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS,
 							NULL);
@@ -2553,7 +2677,8 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 
 			if (entry != NULL) {
 				/* FIXME: Handle name conflicts? */
-				nautilus_link_local_create_from_gnome_entry (entry, container_uri, &point);
+				nautilus_link_local_create_from_gnome_entry (entry, container_uri, 
+				                                             (n_links > 0) ? NULL: &point, screen_num);
 
 				gnome_desktop_item_unref (entry);
 				continue;
@@ -2573,11 +2698,21 @@ icon_view_handle_uri_list (NautilusIconContainer *container, const char *item_ur
 				/* FIXME: Handle name conflicts? */
 				nautilus_link_local_create (container_uri, link_name,
 							    NULL, uri,
-							    &point, NAUTILUS_LINK_GENERIC);
+							    (n_links > 0) ? NULL: &point, 
+							    screen_num,
+							    NAUTILUS_LINK_GENERIC);
 			}
 			
+			n_links++;
 			g_free (stripped_uri);
-			break;
+			
+			/* The following break statement handles mozilla urls.  
+			 * Do not remove it; otherwise, nautilus will create two 
+			 * links when the url is dropped on the desktop.
+			 */
+			if (path == NULL) {
+				break;
+			}
 		}
 	}
 	
@@ -2643,6 +2778,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	fm_directory_view_class->get_background_widget = fm_icon_view_get_background_widget;
 	fm_directory_view_class->get_selected_icon_locations = fm_icon_view_get_selected_icon_locations;
 	fm_directory_view_class->get_selection = fm_icon_view_get_selection;
+	fm_directory_view_class->get_item_count = fm_icon_view_get_item_count;
 	fm_directory_view_class->is_empty = fm_icon_view_is_empty;
 	fm_directory_view_class->remove_file = fm_icon_view_remove_file;
 	fm_directory_view_class->reset_to_defaults = fm_icon_view_reset_to_defaults;
@@ -2665,6 +2801,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	klass->clean_up = fm_icon_view_real_clean_up;
 	klass->supports_auto_layout = real_supports_auto_layout;
 	klass->supports_keep_aligned = real_supports_keep_aligned;
+	klass->supports_labels_beside_icons = real_supports_labels_beside_icons;
         klass->get_directory_auto_layout = fm_icon_view_real_get_directory_auto_layout;
         klass->get_directory_sort_by = fm_icon_view_real_get_directory_sort_by;
         klass->get_directory_sort_reversed = fm_icon_view_real_get_directory_sort_reversed;
@@ -2720,6 +2857,9 @@ fm_icon_view_instance_init (FMIconView *icon_view)
 	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
 						  default_zoom_level_changed_callback,
 						  icon_view, G_OBJECT (icon_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS,
+						  labels_beside_icons_changed_callback,
+						  icon_view, G_OBJECT (icon_view));
 
 	g_signal_connect_object (get_icon_container (icon_view), "handle_uri_list",
 				 G_CALLBACK (icon_view_handle_uri_list), icon_view, 0);
