@@ -142,7 +142,7 @@ nautilus_bookmarks_window_response_callback (GtkDialog *dialog,
 			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
 							     GTK_DIALOG_DESTROY_WITH_PARENT,
 							     GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_CLOSE,
+							     GTK_BUTTONS_OK,
 							     _("There was an error displaying help: \n%s"),
 							     error->message);
 
@@ -783,12 +783,12 @@ repopulate (void)
 				selection_changed_id);
 	g_signal_handler_block (bookmark_list_store,
 				row_deleted_signal_id);
-        g_signal_handler_block (bookmark_list_store,
+        g_signal_handler_block (bookmark_list_widget,
                                 row_activated_signal_id);
 
 	gtk_list_store_clear (store);
 	
-	g_signal_handler_unblock (bookmark_selection,
+	g_signal_handler_unblock (bookmark_list_widget,
 				  row_activated_signal_id);
 	g_signal_handler_unblock (bookmark_list_store,
 				  row_deleted_signal_id);
